/*
 * Copyright 1993, 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
#include "gl.h"
#include "gl/device.h"
#include "gl/image.h"
#include "math.h"

/* 	cc weather.c -o weather -lgutil -limage -lgl_s -lm */

static float LModel[] = {
    AMBIENT, .10, .11, .12,
    LOCALVIEWER, 0.0,
    ATTENUATION, 1.0, 0.0,
    LMNULL,
};

static float Material[] = {
    SHININESS, 20,
    EMISSION, 0, 0, 0,
    AMBIENT, .10, .11, .12,
    SPECULAR, .50, .51, .52,
    DIFFUSE, 0.369, 0.0, 0.165,
    LMNULL,
};

static float LightOne[] = {
    POSITION, 1.0, 1.0, 1.0, 1.0, /* local light, w=1 */
    LCOLOR, .4, .4, .4,
    AMBIENT, .1, .1, .1,
    LMNULL,
};


Matrix Imat    = {1.,0.,0.,0.,
		 0.,1.,0.,0.,
		 0.,0.,1.,0.,
		 0.,0.,0.,1.};

Matrix Rpxmat,Rpymat,Rmxmat,Rmymat;

Matrix tmat;
float tlutps[] = {TL_NULL};

long winxsize, winysize;
int xorg, yorg;

float myangle = 0;
float zoom = 1.0;
int model = 0;
int lut = 0;
int fov = 50;
float xrot=0.,yrot=0.,ztrans=0.;
int lx, ly, mx, my;


main(argc,argv)
int argc;
char **argv;
{
    short val;
    int lx, ly, mx, my;
    float fx, fy;
    int i;
    unsigned long *image;

    keepaspect(1024,768);
    prefsize(1024,768);
    winopen("weather");
    winconstraints();
    doublebuffer();
    RGBmode();
    gconfig();

    subpixel(TRUE);
    linesmooth(SML_ON|SML_SMOOTHER);

    mmode(MSINGLE);
    window(-.61*1421.43,.88 * 1421.43,.27 * 1421.43,1421.43,10000.,26370.);
    lookat(0.,0.,26370.,0.,0.,0.,0.);
    getmatrix(tmat); 
    loadmatrix(Imat);
    scale(.5,.5,1.);
    translate(1.,1.,0.);
    multmatrix(tmat);
    getmatrix(tmat);


    mmode(MTEXTURE);
    loadmatrix(tmat);

    mmode(MVIEWING);
    perspective(fov,1024./768.,100.,106370.); 
    loadmatrix(Imat);
    lookat(0.,0.,1.5*26370.,0.,0.,0.,0.);
    scale(.5,.5,.5);
    cpack(0xffffffff);

    lmdef(DEFLMODEL, 1, sizeof(LModel)/sizeof(float), LModel);
    lmdef(DEFMATERIAL, 1, sizeof(Material)/sizeof(float), Material);
    lmdef(DEFLIGHT, 1, sizeof(LightOne)/sizeof(float), LightOne);

    lmbind(LMODEL, 1);
    lmbind(MATERIAL, 1);
    lmbind(LIGHT0, 1);
    lmcolor(LMC_COLOR);
    shademodel(FLAT);

    cpack(0xffff0000);
    clear();

    {
	 float texps[] = {TX_MAGFILTER,TX_BILINEAR,TX_MINFILTER,TX_BILINEAR,TX_WRAP,TX_CLAMP,0};
	 unsigned long tex[512*1024];
         image = (unsigned long *) longimagedata("weather1024.rgb");
	 loadtexture(image,tex,1,1024,512);
   	 texdef2d(1,1,1024,512,tex, 0,texps);
    }
    {
	float tevps[2] = {TV_MODULATE,0.};
	tevdef(1,0,tevps);
    }
    texbind(0,1);
    tevbind(0,1);

    makemesh1(-1.5,1.5,-1.5,1.5,0.0,1.0,0.0,1.0,0.0,64,64);

    makeframe();
    maketable1();
    maketable2();
    maketable3();
    maketable4();

    qdevice(RIGHTARROWKEY);
    qdevice(LEFTARROWKEY);
    qdevice(ESCKEY);
    qdevice(LEFTMOUSE);
    qdevice(MIDDLEMOUSE);
    qdevice(RIGHTMOUSE);

    drawtiles();
    swapbuffers();

    while(1) {
	while (qtest()) {
	    int dev;
	    
	    dev = qread(&val);
	    if (val)
	switch(dev) {
	    case REDRAW:
		makeframe();
		break;
	    case ESCKEY:
		exit(0);
		break;
	    case LEFTARROWKEY:
		lut = (lut + 1) % 5;
   		tlutbind(0,lut);
		break;
	    case RIGHTARROWKEY:
		model++;
		switch(model%2) {
		case 0:
    		    makemesh0(-1.5,1.5,-1.5,1.5,0.0,1.0,0.0,1.0,0.0,64,64);
		    break;
		case 1:
    		    makemesh1(-1.5,1.5,-1.5,1.5,0.0,1.0,0.0,1.0,0.0,64,64);
		    break;
		}
		break;
	    case LEFTMOUSE:
		if(val) {
		    lx = getmousex(); 
		    ly = getmousey(); 
		    while(getbutton(LEFTMOUSE)) {
			mx = getmousex(); 
			my = getmousey(); 
			if(mx!=lx || my!=ly) {
			    yrot = (mx-lx)*(1./34.);
			    xrot = -(my-ly)*(1./34.);
		  	    mmode(MTEXTURE);
			    rot(xrot,'x');
			    rot(yrot,'y');
			    drawtiles(); 
			    swapbuffers();
			    lx = mx;
			    ly = my;
			}
		    }
		}
		break;
	    case MIDDLEMOUSE:
		if(val) {
		    ly = getmousey(); 
		    while(getbutton(MIDDLEMOUSE)) {
			my = getmousey(); 
			if(my!=ly) {
			    float sf; 
			    if (my > ly)
				sf = 1. + .005 * (my-ly);
			    else
				sf = 1./(1.+ .005 * (ly-my));
			    scale(sf,sf,sf);
			    drawtiles(); 
			    swapbuffers();
			    ly = my;
			}
		    }
		}
		break;
	    case RIGHTMOUSE:
		if(val) {
		    lx = getmousex(); 
		    ly = getmousey(); 
		    while(getbutton(RIGHTMOUSE)) {
			mx = getmousex(); 
			my = getmousey(); 
			if(mx!=lx || my!=ly) {
			    yrot = (mx-lx)*(1./34.);
			    xrot = -(my-ly)*(1./34.);
			    rot(xrot,'x');
			    rot(yrot,'y');
			    drawtiles(); 
			    swapbuffers();
			    lx = mx;
			    ly = my;
			}
		    }
		}
		break;
	}
	}
	drawtiles(); 
	xrot = yrot = 0;
	swapbuffers();
    }
}


makeframe()
{
    reshapeviewport();
    getsize(&winxsize,&winysize);
    drawtiles(winxsize,winysize);
}


float N = 2.0;
float B = -1.5;

#define MAXMESH 64

float Ml[4*(MAXMESH+1)*2 * (MAXMESH+1)];
float TMl[4*(MAXMESH+1)*2 * (MAXMESH+1)];
float Tl[4*(MAXMESH+1)*2 * (MAXMESH+1)];


makemesh1(x0,x1,y0,y1,s0,s1,t0,t1,z,nx,ny)
float x0,x1,y0,y1,s0,s1,t0,t1,z;
int nx,ny;
{
	float y,x,s,t,dx,dy,ds,dt,vb[3],tb[2];
	float v,q;
	float *mp = Ml;
	float *tp = TMl;
	float radius,dradius,middle,px,py,dpx,dpy;
	int i,j;

	radius = 4000.0;

	ds = (s1-s0)/nx;
	dt = (t1-t0)/ny;

	y =  -M_PI/2.0;
	dy = M_PI/ny;
	dx = M_PI/nx;	

	for (i=0; i < ny; i++ ) {
		x = M_PI;
		s = s0;
		for (j=0; j <= nx; j++) {
			vb[0] = cosf(x)*radius* cosf(y); vb[1] = sinf(y)*radius; vb[2] = z - sinf(x)*radius*cosf(y);
			*tp++ = vb[0];	
			*tp++ = vb[1];	
			*tp++ = vb[2];
			*tp++ = 1;
			*mp++ = vb[0];	
			*mp++ = vb[1];	
			*mp++ = vb[2];	
			*mp++ = 1.;
			vb[0] = cosf(x)*radius* cosf(y+dy); vb[1] = sinf(y+dy)*radius; vb[2] = z - sinf(x)*radius*cosf(y+dy);
			*tp++ = vb[0];	
			*tp++ = vb[1];	
			*tp++ = vb[2];
			*tp++ = 1;
			*mp++ = vb[0];	
			*mp++ = vb[1];	
			*mp++ = vb[2];	
			*mp++ = 1.;
			x += dx;
			s += ds;
		}	
		y += dy;
		t += dt;
	}
}

makemesh0(x0,x1,y0,y1,s0,s1,t0,t1,z,nx,ny)
float x0,x1,y0,y1,s0,s1,t0,t1,z;
int nx,ny;
{
	float y,x,s,t,dx,dy,ds,dt,vb[3],tb[2];
	float v,q;
	float *mp = Ml;
	float radius,dradius,middle,px,py,dpx,dpy;
	int i,j;

	radius = 4000.0;

	y =  -radius;
	dy = 2*radius/ny;
	dx = 2*radius/nx;	

	for (i=0; i < ny; i++ ) {
		x = -radius;
		for (j=0; j <= nx; j++) {
			vb[0] = x; vb[1] = y; vb[2] = z;
			*mp++ = vb[0];	
			*mp++ = vb[1];	
			*mp++ = vb[2];	
			*mp++ = 1.;
			vb[1] = y+dy; 
			*mp++ = vb[0];	
			*mp++ = vb[1];	
			*mp++ = vb[2];	
			*mp++ = 1.;
			x += dx;
		}	
		y += dy;
	}
}


makemesh2(x0,x1,y0,y1,s0,s1,t0,t1,z,nx,ny)
float x0,x1,y0,y1,s0,s1,t0,t1,z;
int nx,ny;
{
	float y,x,s,t,dx,dy,ds,dt,vb[3],tb[2];
	float v,q;
	float *mp = Ml;
	float *tp = Tl;
	float radius,dradius,middle,px,py,dpx,dpy;
	int i,j;

	radius = 4000.0;

	ds = (s1-s0)/nx;
	dt = (t1-t0)/ny;

	y =  -M_PI/2.0;
	dy = M_PI/ny;
	dx = M_PI/nx;	

	for (i=0; i < ny; i++ ) {
		x = M_PI;
		s = s0;
		for (j=0; j <= nx; j++) {
			vb[0] = cosf(x)*radius* cosf(y*.75); vb[1] = sinf(y)*radius; vb[2] = z - sinf(x)*radius*cosf(y*.75);
			*mp++ = vb[0];	
			*mp++ = vb[1];	
			*mp++ = vb[2];	
			*mp++ = 1.;
			vb[0] = cosf(x)*radius* cosf((y+dy)*.75); vb[1] = sinf(y+dy)*radius; vb[2] = z - sinf(x)*radius*cosf((y+dy)*.75);
			*mp++ = vb[0];	
			*mp++ = vb[1];	
			*mp++ = vb[2];	
			*mp++ = 1.;
			x += dx;
			s += ds;
		}	
		y += dy;
		t += dt;
	}
}


drawmesh(nx,ny)
int nx,ny;
{
	float *mp = Ml;
	float *tp = TMl;
	int i,j;

	for (i=ny+1; i; i--) {
		bgntmesh();
		for (j = nx+1; j; j--) {
			t4f(tp); tp += 4;
			v3f(mp); mp += 4;
			t4f(tp); tp += 4;
			v3f(mp); mp += 4;
		}
		endtmesh();
	}
}

drawlines(nx,ny)
int nx,ny;
{
	float *mp = Ml;
	int i,j,delta;

	delta = (nx+1)*8;
	for (i=ny+1; i; i--) {
		bgnline();
		for (j = nx+1; j; j--) {
			v3f(mp); mp += 8;
		}
		endline();
	}
	for (i=0; i < nx+1; i++) {
		mp = Ml+8*i;
		bgnline();
		for (j = ny; j; j--) {
			v3f(mp); mp += delta;
		}
		endline();
	}
}




drawtiles()
{
    int minx,maxx,miny,maxy;
    float s0,s1,t0,t1;
    int itmp,first,last;
    int x,y;

    cpack(0xffff0000);
    clear();
    mmode(MVIEWING);

    texbind(0,1);
    cpack(0xffffffff);
    drawmesh(64,64);
    texbind(0,0);
    blendfunction(BF_SA,BF_MSA);
    cpack(0x80ffffff);
    drawlines(64,64);
    blendfunction(BF_ONE,BF_ZERO);
    
}

loadtexture(image,tex,nc,w,h)
unsigned long *image,*tex;
long nc,w,h;
{
	unsigned char *bw;
	int r,g,b,a;
	int i,j,skip;

	switch(nc) {
		case 1:	/* bw */
			skip = (4-(w % 4)) % 4;
			bw = (unsigned char *) tex;
			for (i=0; i < h; i++) {
				for(j=0; j<w; j++) 
					*bw++ =	 *image++ & 0xff;
				bw += skip;
			}
			break;
		case 2:	/* IA */
			skip = 2*(w % 2);
			bw = (unsigned char *) tex;
			for (i=0; i < h; i++) {
				for (j=0; j < w; j++) {
					r = *image++ & 0xff; 
					a = r;
					*bw++ = a;
					*bw++ = r;
				}
				bw += skip;
			}
			break;
		case 3:	/* RGB */
			skip = (w % 4);
			bw = (unsigned char *) tex;
			for (i=0; i < h; i++) {
				for (j=0; j < w; j++) {
					r = (*image) & 0xff; 
					g = (*image >> 8) & 0xff; 
					b = (*image >> 16) & 0xff; 
					image++;
					*bw++ = b;
					*bw++ = g;
					*bw++ = r;
				}
				bw += skip;
			}
			break;
		case 4:	/* RGBA */
			bw = (unsigned char *) tex;
			for (i=0; i < h; i++) {
				for (j=0; j < w; j++) {
					r = (*image) & 0xff; 
					g = (*image >> 8) & 0xff; 
					b = (*image >> 16) & 0xff; 
					a = r;
					image++;
					*bw++ = a;
					*bw++ = b;
					*bw++ = g;
					*bw++ = r;
				}
			}
			break;
	}
}

maketable4()
{    
   int i;
   unsigned long table[256];

   for (i = 0; i < 256; i++){
      table[i] = ((255-i)<<24) | ((255-i)<<16) | ((255-i)<<8) | ((255-i)); 
   }
   tlutdef(4,4,256,table,0,tlutps);
}

maketable1()
{    
   int i;
   unsigned long table[256];
   float v;
   int iv;

   for (i = 0; i < 256; i++){
      v = i/255.0;
      iv = powf(v,(float) 1.5) * 255;
      table[i] = ((iv)<<24) | ((iv)<<16) | ((iv)<<8) | ((iv)); 
   }
   tlutdef(1,4,256,table,0,tlutps);
}

maketable2()
{    
   int i;
   unsigned long table[256];
   float v;
   int iv;

   for (i = 0; i < 256; i++){
      v = i/255.0;
      iv = powf(v,(float) 1.0) * 255;
      table[i] = ((iv)<<24) | ((iv)<<16) | ((iv)<<8) | ((iv)); 
   }
   tlutdef(2,4,256,table,0,tlutps);
}

maketable3()
{    
   int i;
   unsigned long table[256];
   float v;
   int iv;

   for (i = 0; i < 256; i++){
      v = i/255.0;
      iv = powf(v,(float) 0.75) * 255;
      table[i] = ((iv)<<24) | ((iv)<<16) | ((iv)<<8) | ((iv)); 
   }
   tlutdef(3,4,256,table,0,tlutps);
}
